/*--------------------------------------------------------------
	Image Processing functions    v2.1
		Written  by H.Goto , Jul 1994
		Modified by H.Goto , Nov 1995
		Modified by H.Goto , May 1996
		Modified by H.Goto , Sep 1997
		Modified by H.Goto , Oct 1999
		Modified by H.Goto , Apr 2000
		Modified by H.Goto , Jul 2000
		Modified by H.Goto , Sep 2000
		Modified by H.Goto , Feb 2002
		Modified by H.Goto , May 2002
--------------------------------------------------------------*/

/*--------------------------------------------------------------------
  Copyright (C) 1994-2002  Hideaki Goto

        All Rights Reserved

  Permission to use, copy, modify, and distribute this software and
  its documentation for any purpose is hereby granted without fee,
  provided that (i) the above copyright notice and this permission
  notice appear in all copies and in supporting documentation, (ii)
  the name of the author, Hideaki Goto, may not be used in any
  advertising or otherwise to promote the sale, use or other
  dealings in this software without prior written authorization
  from the author, (iii) this software may not be used for
  commercial products without prior written permission from the
  author, and (iv) the notice of modification is specified in cases
  where modified copies of this software are distributed.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
  THE AUTHOR WILL NOT BE RESPONSIBLE FOR ANY DAMAGE CAUSED BY THIS
  SOFTWARE.
--------------------------------------------------------------------*/


#ifdef	__cplusplus
extern "C" {
#endif

#ifndef siplib_h
#define siplib_h


#include	<utypes.h>	/* define unsigned variables */



/*------------------------------------------------------
	Common constants
------------------------------------------------------*/

#define	BOP_NOP		0
#define	BOP_PUT		1
#define	BOP_OR		2
#define	BOP_NOT		3
#define	BOP_AND		4
#define	BOP_XOR		5
#define	BOP_SET		6
#define	BOP_CLR		7




/*------------------------------------------------------
	Structure definitions for Mask-Op.
------------------------------------------------------*/

typedef struct {
	int	masksize;	/* odd integer required    */
				/*   ex.  5 for 5x5 matrix */
	ushort	depth;		/* = 16 */
	short	div;		/* normalize factor */
	short	*mask;
} SIPMASKs;


typedef struct {
	int	masksize;
	ushort	depth;		/* = 32 */
	int32	div;
	int32	*mask;
} SIPMASKl;


typedef struct {
	int	masksize;
	ushort	depth;		/* = 32 | 0x8000 */
	float	div;
	float	*mask;
} SIPMASKf;


typedef union {
	SIPMASKs	s;
	SIPMASKl	l;
	SIPMASKf	f;
} SIPMASK;




/*------------------------------------------------------
	Primitives
------------------------------------------------------*/

typedef struct {			/* Rectangle */
	short	x,y;
	ushort	width,height;
} SIPRectangle;


typedef struct {			/* Line Segment */
	short	x1,y1,x2,y2;
} SIPSegment;




/*------------------------------------------------------
	Image object
------------------------------------------------------*/

typedef struct {			/* Image (Array) Object */
	ushort	width,height;
	ushort	depth;			/* 1,8,16,32 */
	ushort	bitmap_pad;		/* 8,16,32 */
	int32	bytes_per_line;
	char	*data;
} SIPImage;


#ifndef siplib_c
extern SIPImage	SIPImage0;		/* Skelton of SIPImage */
#endif


extern	SIPImage*	sip_CreateImage(ushort width,ushort height,ushort depth);
extern	int	 	sip_DestroyImage(SIPImage *image);




/*------------------------------------------------------
	Image Data Manipulation
------------------------------------------------------*/

extern	int		sip_rawClearLine(char *buf,ushort len,ushort depth,int32 data);
extern	int		sip_ClearLine(SIPImage *image,ushort y,int32 data);
extern	int		sip_ClearImage(SIPImage *image,int32 data);
extern	int		sip_CopyArea(SIPImage *src,SIPRectangle *srect,SIPImage *dst,SIPRectangle *drect);
extern	int		sip_GetScanLine(SIPImage *image,void *buf,short x,short y,short len,ushort depth);
extern	int		sip_PutScanLine(SIPImage *image,void *buf,short x,short y,short len,ushort depth);
extern	int		sip_GetVertLine(SIPImage *image,void *buf,short x,short y,short len,ushort depth);
extern	int		sip_PutVertLine(SIPImage *image,void *buf,short x,short y,short len,ushort depth);
extern	uchar		sip_getbit(SIPImage *image,ushort x,ushort y);
extern	uchar		sip_putbit(SIPImage *image,ushort x,ushort y,uchar b);
extern	uchar		sip_orbit(SIPImage *image,ushort x,ushort y,uchar b);
extern	uchar		sip_andbit(SIPImage *image,ushort x,ushort y,uchar b);
extern	uchar		sip_xorbit(SIPImage *image,ushort x,ushort y,uchar b);

#define	_sip_getbit(image,x,y) \
	(uchar)(( *((image)->data + (int32)(( (ushort)(x) )>>3) \
		+ (int32)((ushort)(y)) * (image)->bytes_per_line) \
		& (0x01 << (7 - ((uint)(x) & 0x07))) ) != 0)




/*------------------------------------------------------
	Miscellaneous
------------------------------------------------------*/

extern	char*		sip_getimgptr(SIPImage *image,ushort y);
extern	ushort		sip_getlinebytes(ushort len,ushort depth,ushort pad);

extern	void		sip_CopyLine(char *src,char *dst,ushort width,ushort depth);




/*------------------------------------------------------
	Data conversion
------------------------------------------------------*/

extern	void		sip_cvt1to8(char *src,ushort bitoff,char *dst,ushort len,char fc,char bc);
extern	void		sip_cvt1to16(char *src,ushort bitoff,short *dst,ushort len,short fc,short bc);
extern	void		sip_cvt1to32(char *src,ushort bitoff,int32 *dst,ushort len,int32 fc,int32 bc);
extern	void		sip_cvt8to16u(uchar *src,ushort *dst,ushort len,ushort mul);
extern	void		sip_cvt8to16(char *src,short *dst,ushort len,short mul);
extern	void		sip_cvt8to32u(uchar *src,uint32 *dst,ushort len,uint32 mul);
extern	void		sip_cvt8to32(char *src,int32 *dst,ushort len,int32 mul);
extern	void		sip_cvt16to32u(ushort *src,uint32 *dst,ushort len,uint32 mul);
extern	void		sip_cvt16to32(short *src,int32 *dst,ushort len,int32 mul);

extern	void		sip_cvt8to1(char *src,char *dst,ushort bitoff,ushort len,char th);
extern	void		sip_cvt8to1u(uchar *src,uchar *dst,ushort bitoff,ushort len,uchar th);
extern	void		sip_cvt16to1(short *src,char *dst,ushort bitoff,ushort len,short th);
extern	void		sip_cvt16to8u(ushort *src,uchar *dst,ushort len,ushort mul,ushort div);



/*------------------------------------------------------
	Clipping
------------------------------------------------------*/

extern	void		clip8(char *src,char *dst,ushort len,char lo,char hi);
extern	void		clip8u(uchar *src,uchar *dst,ushort len,uchar lo,uchar hi);
extern	void		clip16(short *src,short *dst,ushort len,short lo,short hi);
extern	void		clip16u(ushort *src,ushort *dst,ushort len,ushort lo,ushort hi);
extern	void		clipint(int *src,int *dst,ushort len,int lo,int hi);
extern	void		clipuint(uint *src,uint *dst,ushort len,uint lo,uint hi);




/*------------------------------------------------------
	Image rotation
------------------------------------------------------*/

extern	int		sip_rotate1(SIPImage *src,SIPImage *dst, \
					int x0,int y0,double angle,int mode);
extern	int		sip_rotate8(SIPImage *src,SIPImage *dst, \
					int x0,int y0,double angle,int mode);



/*------------------------------------------------------
	IP-Stream
------------------------------------------------------*/

#include	"ipstream.h"





/*------------------------------------------------------
	Some useful functions
------------------------------------------------------*/

#ifdef siplib_c
static int		sip_convolution3x3s(SIPMASKs *mask,IPStream *ips,short *buf);
#endif

extern	int		sip_convolution_s(SIPMASKs *mask,IPStream *ips,short *buf);
extern	int		sip_convolution(SIPMASK *mask,IPStream *ips,char *buf);

extern	int		sip_thin10neib(SIPImage *image,char *cflag,char *rflag);
extern	int		sip_distimage(SIPImage *image,SIPImage *distimage,int maxdist);

extern	long		sip_projprofile(SIPImage *image,long *rden,long *cden);
extern	long		sip_projprofile_area(SIPImage *image, \
				long *rden,long *cden,SIPRectangle *cbox);

extern	int		sip_kNNsmooth(SIPImage *src,SIPImage *dst,int pixels,int ctweight);


#endif	/* siplib_h */

#ifdef	__cplusplus
}
#endif
