/*--------------------------------------------------------------
        Image Processing function library    libsip
	  (IP-Stream control)
		Written       by H.Goto , Jul 1994
		Last modified by H.Goto , May 1996
		Last modified by H.Goto , Apr 2000
		Last modified by H.Goto , Feb 2002
--------------------------------------------------------------*/

/*--------------------------------------------------------------------
  Copyright (C) 1994-2002  Hideaki Goto

        All Rights Reserved

  Permission to use, copy, modify, and distribute this software and
  its documentation for any purpose is hereby granted without fee,
  provided that (i) the above copyright notice and this permission
  notice appear in all copies and in supporting documentation, (ii)
  the name of the author, Hideaki Goto, may not be used in any
  advertising or otherwise to promote the sale, use or other
  dealings in this software without prior written authorization
  from the author, (iii) this software may not be used for
  commercial products without prior written permission from the
  author, and (iv) the notice of the modification is specified in
  case of that the modified copies of this software are distributed.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
  THE AUTHOR WILL NOT BE RESPONSIBLE FOR ANY DAMAGE CAUSED BY THIS
  SOFTWARE.
--------------------------------------------------------------------*/


#include	<stdio.h>
#include	<stdlib.h>
#include	<memory.h>

#include	"siplib.h"


/*------------------------------------------------------
	IP-Stream
------------------------------------------------------*/

				/* Get ptr. for input line-buffer */

char *sip_StGetInputBuffer(IPStream *ips){
	return ( ips->line[0] + ips->guardsize * (ips->depth >> 3) );
}


				/* Rotate Scanline ptr. */

void sip_StRotate(char **line,ushort lines){
	char	*lnsave;
	ushort	i;
	if ( lines ){
		lnsave = line[0];
		for ( i=1 ; i<lines ; i++ )  line[i-1] = line[i];
		line[lines-1] = lnsave;
	}
}


				/* Create IP-Stream structure */

IPStream *sip_CreateIPStream(ushort width,ushort height,ushort depth, \
				ushort guardsize,int32 guarddata){
	int		i;
	IPStream	*ips;
	ushort		bandsize;
	switch (depth){
	  case 8:
	  case 16:
	  case 32:	break;
	  default:	return(NULL);
	}
	if ( (width == 0) || (height == 0) ){
		return(NULL);
	}
	if ( NULL == (ips = (IPStream *)malloc(sizeof(IPStream)) ) ){
		return(NULL);
	}
	ips->status = 0;
	ips->in_count = ips->out_count = 0;
	ips->next = ips->prev = NULL;
	ips->width  = width;
	ips->height = height;
	ips->depth  = depth;
	ips->guardsize = guardsize;
	ips->guarddata = guarddata;
	ips->line = NULL;
	ips->tmpline = NULL;
	bandsize = guardsize * 2 + 1;
	if ( guardsize != 0 ){
		if ( NULL == (ips->line = (char **)malloc(bandsize * sizeof(char *))) ){
			free((char *)ips);
			return(NULL);
		}
		if ( NULL == (ips->tmpline = sip_CreateImage((ushort)(width + guardsize * 2),bandsize,depth)) ){
			free((char *)ips->line);
			free((char *)ips);
			return(NULL);
		}
		for( i=0 ; i<bandsize ; i++ ){
			ips->line[i] = (char *)sip_getimgptr(ips->tmpline,(ushort)i);
		}
		sip_ClearImage(ips->tmpline,guarddata);
	}


	return(ips);
}


				/* Destroy IP-Stream structure */

int sip_DestroyIPStream(IPStream *ips){
	if ( NULL != ips ){
		if ( NULL != ips->tmpline ){
			if ( sip_DestroyImage(ips->tmpline) )  return(-1);
			ips->tmpline = NULL;
		}
		if ( NULL != ips->line ){
			free((char *)ips->line);
			ips->line = NULL;
		}
		free((char *)ips);
	}
	return(0);
}


				/* Put Dummy Line into IP-Stream */

int sip_StPutDummy(IPStream *ips){
	char	*buf;
	buf = sip_StGetInputBuffer(ips);
	sip_rawClearLine(buf,ips->width,ips->depth,ips->guarddata);
	sip_StRotate(ips->line,(ushort)(ips->guardsize *2 +1));
	++ips->in_count;
	ips->status &= ~StSTAT_IDRQ;
	ips->status |= StSTAT_IDSR;
	return(0);
}


				/* Put ScanLine into IP-Stream */

int sip_StPutScanLine(IPStream *ips,void *buf){
	sip_CopyLine((char *)buf,sip_StGetInputBuffer(ips),ips->width,ips->depth);
	sip_StRotate(ips->line,(ushort)(ips->guardsize *2 +1));
	++ips->in_count;
	ips->status &= ~StSTAT_IDRQ;
	ips->status |= StSTAT_IDSR;
	return(0);
}


#if 0
				/* Discard a line */

int sip_StDiscardLine(IPStream *ips){
	ips->status &= ~StSTAT_HOLD;
	return(0);
}
#endif

				/* Set DSR  and  Pass it to the next stream */

void sip_StSetDSR(IPStream *ips){
	++ips->out_count;
	if ( ips->next == NULL )  ips->status |= StSTAT_ODSR;
	else  ips->next->status |= StSTAT_IDSR;
}


				/* Reset DRQ  and  Pass it to the next stream */

void sip_StResetDRQ(IPStream *ips){
	ips->status &= ~StSTAT_ODRQ;
	if ( ips->next != NULL )  ips->next->status &= ~StSTAT_IDRQ;
}


				/* Check DRQ */

int sip_StCheckDRQ(IPStream *ips){
	if ( ips->next == NULL )  return(ips->status & StSTAT_ODRQ);
	else  return( ips->next->status & StSTAT_IDRQ );
}


				/* Create pipe */

int sip_StCreatePipe(IPStream *ips,IPStream *ips_next){
	if ( ips->next != NULL )  return(-1);
	if ( ips_next->prev != NULL )  return(-2);
	ips->next = ips_next;
	ips_next->prev = ips;
	return(0);
}


				/* Destroy pipe */

int sip_StDestroyPipe(IPStream *ips){
	if ( NULL == ips->next )  return(-1);
	ips->next->prev = ips->next = NULL;
	return(0);
}


#if 0
				/* Open pipe */

char *sip_StOpenPipe(IPStream *ips){
	if ( NULL == ips->next )  return(NULL);
	ips->status |= StSTAT_IOBUSY;	 	/* Set busy-flag (option) */
	return( sip_StGetInputBuffer(ips->next) );
}


				/* Close pipe */

int sip_StClosePipe(IPStream *ips){
	if ( NULL == ips->next )  return(-1);
	sip_StRotate(ips->next->line,ips->next->guardsize *2 +1);
	++ips->next->count;
	ips->next->status &= ~StSTAT_IOBUSY;	/* Reset busy-flag (option) */
	ips->next->status |= StSTAT_HOLD;
	return(0);
}
#endif



